<?php

namespace Tell\Reader;

class JsonReader implements ReaderInterface {
    
    protected string $name = "json";
    
    protected array $accept = [ "json" ];
    
    protected int $saveFlag = 0;
    
    public array $pathList;
    
    public bool $filenameAsKey = false;
    
    public function __construct(array $pathList, bool $filenameAsKey = false, int $saveFlag = \JSON_PRETTY_PRINT)
    {
        $this->pathList = $pathList;
        $this->filenameAsKey = $filenameAsKey;
        $this->saveFlag = $saveFlag;
    }

    public function accept(string $extension) : bool
    {
        return in_array($extension, $this->accept);
    }
    
    public function load(string $filepath) : ?array 
    {
        if ( ! file_exists($filepath) ) {
            throw new \Exception("Given file path is not existing or is inaccessible.");
        }
        
        if ( null === ( $content = json_decode(file_get_contents($filepath), true) ) ) {
            throw new \Exception("An error occured trying to parse configuration file: given json file [ $filepath ] seems to be invalid");
        }

        return $content;
    }

    public function save(string $filepath, array $content) : bool
    {
        $path = $this->pathList[0];
        
        if ( !file_exists($path) ) {
            mkdir($path, 0750, true);
        } 
        
        return ( file_put_contents($path . DIRECTORY_SEPARATOR . "$filepath." . $this->accept[0], json_encode($content, $this->saveFlag)) !== false ); 
    }
}
